@@ -45,7 +45,7 @@ from jdaviz.core.events import (LoadDataMessage, NewViewerMessage, AddDataMessag
 from jdaviz.core.registries import (tool_registry, tray_registry, viewer_registry,
                                     data_parser_registry)
 from jdaviz.core.tools import ICON_DIR
-from jdaviz.utils import SnackbarQueue
+from jdaviz.utils import SnackbarQueue, ColorCycler
 from ipypopout import PopoutButton
 
 __all__ = ['Application']
@@ -68,6 +68,7 @@ custom_components = {'j-tooltip': 'components/tooltip.vue',
                      'j-docs-link': 'components/docs_link.vue',
                      'j-viewer-data-select': 'components/viewer_data_select.vue',
                      'j-viewer-data-select-item': 'components/viewer_data_select_item.vue',
+                     'j-layer-viewer-icon': 'components/layer_viewer_icon.vue',
                      'j-tray-plugin': 'components/tray_plugin.vue',
                      'j-play-pause-widget': 'components/play_pause_widget.vue',
                      'j-plugin-section-header': 'components/plugin_section_header.vue',
@@ -146,8 +147,8 @@ class ApplicationState(State):
         'checktoradial': read_icon(os.path.join(ICON_DIR, 'checktoradial.svg'), 'svg+xml')
     }, docstring="Custom application icons")
 
-    viewer_icons = DictCallbackProperty({}, docstring="Indexed icons for viewers across the app")
-    layer_icons = DictCallbackProperty({}, docstring="Indexed icons for layers across the app")
+    viewer_icons = DictCallbackProperty({}, docstring="Indexed icons (numbers) for viewers across the app")  # noqa
+    layer_icons = DictCallbackProperty({}, docstring="Indexed icons (letters) for layers across the app")  # noqa
 
     data_items = ListCallbackProperty(
         docstring="List of data items parsed from the Glue data collection.")
@@ -378,7 +379,7 @@ class Application(VuetifyTemplate, HubListener):
 
         if layer_name not in self.state.layer_icons:
             self.state.layer_icons = {**self.state.layer_icons,
-                                      layer_name: f"mdi-alpha-{chr(97 + len(self.state.layer_icons))}-box-outline"}  # noqa
+                                      layer_name: chr(97 + len(self.state.layer_icons))}
 
     def _link_new_data(self, reference_data=None, data_to_be_linked=None):
         """
@@ -403,17 +404,16 @@ class Application(VuetifyTemplate, HubListener):
         ref_data = dc[reference_data] if reference_data else dc[0]
         linked_data = dc[data_to_be_linked] if data_to_be_linked else dc[-1]
 
-        if linked_data.meta.get('Plugin', None) == 'SpectralExtraction':
-            if 'Trace' in linked_data.meta:
-                links = [LinkSame(linked_data.components[2], ref_data.components[0]),
-                         LinkSame(linked_data.components[0], ref_data.components[1])]
-                dc.add_link(links)
-                return
-            else:
-                links = [LinkSame(linked_data.components[0], ref_data.components[0]),
-                         LinkSame(linked_data.components[1], ref_data.components[1])]
-                dc.add_link(links)
-                return
+        if 'Trace' in linked_data.meta:
+            links = [LinkSame(linked_data.components[1], ref_data.components[0]),
+                     LinkSame(linked_data.components[0], ref_data.components[1])]
+            dc.add_link(links)
+            return
+        elif linked_data.meta.get('Plugin', None) == 'SpectralExtraction':
+            links = [LinkSame(linked_data.components[0], ref_data.components[0]),
+                     LinkSame(linked_data.components[1], ref_data.components[1])]
+            dc.add_link(links)
+            return
 
         # The glue-astronomy SpectralCoordinates currently seems incompatible with glue
         # WCSLink. This gets around it until there's an upstream fix.
@@ -474,14 +474,24 @@ class Application(VuetifyTemplate, HubListener):
     def load_data(self, file_obj, parser_reference=None, **kwargs):
         """
         Provided a path to a data file, open and parse the data into the
-        `~glue.core.data_collection.DataCollection` for this session. This also attempts to
-        find WCS links that exist between data components. Extra key word
-        arguments are passed to the parsing functions.
+        `~glue.core.data_collection.DataCollection` for this session.
+
+        For some parsers, this also attempts to find WCS links that exist
+        between data components.
 
         Parameters
         ----------
         file_obj : str or file-like
             File object for the data to be loaded.
+
+        parser_reference : str or `None`
+            The actual data parser to use. It must already be registered
+            to glue's data parser registry. This is mainly for internal use.
+
+        **kwargs : dict
+            Additional keywords to be passed into the parser defined by
+            ``parser_reference``.
+
         """
         self.loading = True
         try:
@@ -1296,6 +1306,8 @@ class Application(VuetifyTemplate, HubListener):
 
         active_data_labels = []
 
+        color_cycler = ColorCycler()
+
         # Include any selected data in the viewer
         for data_id, visibility in selected_items.items():
             label = self._get_data_item_by_id(data_id)['name']
@@ -1309,7 +1321,7 @@ class Application(VuetifyTemplate, HubListener):
 
             [data] = [x for x in self.data_collection if x.label == label]
 
-            viewer.add_data(data, percentile=95)
+            viewer.add_data(data, percentile=95, color=color_cycler())
 
             add_data_message = AddDataMessage(data, viewer,
                                               viewer_id=viewer_id,
@@ -1321,6 +1333,7 @@ class Application(VuetifyTemplate, HubListener):
                     if visibility == 'visible' and not layer.visible:
                         layer.visible = True
                         layer.update()
+
                     else:
                         layer.visible = visibility == 'visible'
 
@@ -1333,6 +1346,8 @@ class Application(VuetifyTemplate, HubListener):
         for data in viewer_data:
             if data.label not in active_data_labels:
                 viewer.remove_data(data)
+                viewer._layers_with_defaults_applied= [layer_info for layer_info in viewer._layers_with_defaults_applied  # noqa
+                                                       if layer_info['data_label'] != data.label]
                 remove_data_message = RemoveDataMessage(data, viewer,
                                                         viewer_id=viewer_id,
                                                         sender=self)
@@ -1497,7 +1512,7 @@ class Application(VuetifyTemplate, HubListener):
         # own attribute instead.
         viewer._reference_id = vid  # For reverse look-up
 
-        self.state.viewer_icons.setdefault(vid, f"mdi-numeric-{len(self.state.viewer_icons)+1}-circle-outline")  # noqa
+        self.state.viewer_icons.setdefault(vid, len(self.state.viewer_icons)+1)
 
         return {
             'id': vid,
@@ -1725,7 +1740,7 @@ class Application(VuetifyTemplate, HubListener):
 
         tray_item = None
         for item in self.state.tray_items:
-            if item['name'] == name:
+            if item['name'] == name or item['label'] == name:
                 ipy_model_id = item['widget']
                 tray_item = widget_serialization['from_json'](ipy_model_id, None)
                 break
