@@ -1,7 +1,9 @@
 import os
 
 import pytest
+from astropy.io import fits
 from astropy.nddata import CCDData
+from numpy.testing import assert_allclose
 
 from jdaviz.configs.cubeviz.plugins.moment_maps.moment_maps import MomentMap
 
@@ -22,12 +24,12 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     mm.n_moment = 0  # Collapsed sum, will get back 2D spatial image
     assert mm.results_label == 'moment 0'
 
-    mm.add_results.viewer.selected = 'mask-viewer'
+    mm.add_results.viewer.selected = 'uncert-viewer'
     mm.vue_calculate_moment()
 
     assert mm.moment_available
     assert dc[1].label == 'moment 0'
-    mv_data = cubeviz_helper.app.get_viewer('mask-viewer').data()
+    mv_data = cubeviz_helper.app.get_viewer('uncert-viewer').data()
     # by default, will overwrite the previous entry (so only one data entry)
     assert len(mv_data) == 1
     assert mv_data[0].label == 'moment 0'
@@ -67,10 +69,6 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     mm.vue_save_as_fits()
     assert os.path.isfile(mm.filename)
 
-    # Does not allow overwrite.
-    with pytest.raises(OSError, match='already exists'):
-        mm.vue_save_as_fits()
-
     mm.n_moment = 1
     assert mm.results_label == 'moment 1'
     assert mm.results_label_overwrite is False
@@ -98,3 +96,36 @@ def test_moment_calculation(cubeviz_helper, spectrum1d_cube, tmpdir):
     assert flux_viewer.label_mouseover.value == '+8.00000e+00 Jy'  # Slice 0 has 8 pixels, this is Slice 1  # noqa
     assert flux_viewer.label_mouseover.world_ra_deg == '204.9997755346'
     assert flux_viewer.label_mouseover.world_dec_deg == '27.0000999998'
+
+
+@pytest.mark.filterwarnings('ignore:No observer defined on WCS')
+def test_write_momentmap(cubeviz_helper, spectrum1d_cube, tmp_path):
+    ''' Test writing a moment map out to a FITS file on disk '''
+
+    # Simulate an existing file on disk to check for overwrite warning
+    test_file = tmp_path / "test_file.fits"
+    test_file_str = str(test_file)
+    existing_sentinel_text = "This is a simulated, existing file on disk"
+    test_file.write_text(existing_sentinel_text)
+
+    cubeviz_helper.load_data(spectrum1d_cube, data_label='test')
+    plugin = cubeviz_helper.plugins['Moment Maps']
+    moment = plugin.calculate_moment()
+
+    # By default, we shouldn't be warning the user of anything
+    assert plugin._obj.overwrite_warn is False
+
+    # Attempt to write the moment map to the same file we created earlier.
+    plugin._obj.filename = test_file_str
+    plugin._obj.vue_save_as_fits()
+
+    # We should get an overwrite warning
+    assert plugin._obj.overwrite_warn is True
+    # and shouldn't have written anything (the file should be in tact)
+    assert test_file.read_text() == existing_sentinel_text
+
+    # Force overwrite the existing file
+    plugin._obj.vue_overwrite_fits()
+
+    # Read back in the file and check that it is equal to the one we calculated
+    assert_allclose(moment.data, fits.getdata(test_file_str))
