@@ -9,7 +9,9 @@ from traitlets import List, Unicode, Bool, observe
 from jdaviz.core.custom_traitlets import FloatHandleEmpty
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
-from jdaviz.core.template_mixin import PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin
+from jdaviz.core.template_mixin import (PluginTemplateMixin, DatasetSelectMixin,
+                                        SelectPluginComponent, AddResultsMixin)
+from jdaviz.core.user_api import PluginUserApi
 
 __all__ = ['GaussianSmooth']
 
@@ -20,12 +22,29 @@ u.add_enabled_units([spaxel])
 
 @tray_registry('g-gaussian-smooth', label="Gaussian Smooth")
 class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
+    """
+    See the :ref:`Gaussian Smooth Plugin Documentation <gaussian-smooth>` for more details.
+
+    Only the following attributes and methods are available through the
+    :ref:`public plugin API <plugin-apis>`:
+
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
+      Dataset to use for computing line statistics.
+    * ``mode`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      Only available for Cubeviz.  Whether to use spatial or spectral smoothing.
+    * :attr:`stddev`:
+      Standard deviation of the gaussian to use for smoothing.
+    * ``add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
+    * :meth:`smooth`
+    """
     template_file = __file__, "gaussian_smooth.vue"
     stddev = FloatHandleEmpty(1).tag(sync=True)
     selected_data_is_1d = Bool(True).tag(sync=True)
     show_modes = Bool(False).tag(sync=True)
-    smooth_modes = List(["Spectral", "Spatial"]).tag(sync=True)
-    selected_mode = Unicode("Spectral").tag(sync=True)
+    mode_items = List().tag(sync=True)
+    mode_selected = Unicode().tag(sync=True)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -41,16 +60,28 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
             self.dataset._viewers = ['spectrum-viewer']
             self.dataset._clear_cache()
 
+        self.mode = SelectPluginComponent(self,
+                                          items='mode_items',
+                                          selected='mode_selected',
+                                          manual_options=['Spectral', 'Spatial'])
+
         # set the filter on the viewer options
         self._update_viewer_filters()
 
-    @observe("dataset_selected", "dataset_items", "stddev", "selected_mode")
+    @property
+    def user_api(self):
+        expose = ['dataset', 'stddev', 'add_results', 'smooth']
+        if self.config == "cubeviz":
+            expose += ['mode']
+        return PluginUserApi(self, expose=expose)
+
+    @observe("dataset_selected", "dataset_items", "stddev", "mode_selected")
     def _set_default_results_label(self, event={}):
         label_comps = []
         if hasattr(self, 'dataset') and (len(self.dataset.labels) > 1 or self.app.config == 'mosviz'):  # noqa
             label_comps += [self.dataset_selected]
         if self.config == "cubeviz":
-            label_comps += [f"{self.selected_mode.lower()}-smooth"]
+            label_comps += [f"{self.mode_selected.lower()}-smooth"]
         else:
             label_comps += ["smooth"]
         label_comps += [f"stddev-{self.stddev}"]
@@ -66,9 +97,9 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
         if self.dataset.selected_dc_item is not None:
             self.selected_data_is_1d = len(self.dataset.selected_dc_item.data.shape) == 1
 
-    @observe("selected_mode")
+    @observe("mode_selected")
     def _update_viewer_filters(self, event={}):
-        if event.get('new', self.selected_mode) == 'Spatial':
+        if event.get('new', self.mode_selected) == 'Spatial':
             # only want image viewers in the options
             self.add_results.viewer.filters = ['is_image_viewer']
         else:
@@ -76,12 +107,61 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
             self.add_results.viewer.filters = ['is_spectrum_viewer']
 
     def vue_apply(self, event={}):
-        if self.selected_mode == 'Spatial':
-            self.apply_spatial_convolution()
-        else:
-            self.apply_spectral_smooth()
+        self.smooth(add_data=True)
 
-    def apply_spectral_smooth(self):
+    def smooth(self, add_data=True):
+        """
+        Smooth according to the settings in the plugin.
+
+        Parameters
+        ----------
+        add_data : bool
+            Whether to add the resulting trace to the application, according to the options
+            defined in the plugin.
+
+        Returns
+        -------
+        spec : `~specutils.Spectrum1D`
+            The smoothed spectrum or data cube
+        """
+        if self.mode_selected == 'Spatial':
+            if self.config != 'cubeviz':
+                raise NotImplementedError("spatial smoothing only supported for Cubeviz")
+            # TODO: in vuetify >2.3, timeout should be set to -1 to keep open
+            #  indefinitely
+            snackbar_message = SnackbarMessage(
+                "Smoothing spatial slices of cube...",
+                loading=True, timeout=0, sender=self)
+            self.hub.broadcast(snackbar_message)
+
+            results = self.spatial_smooth()
+
+        else:
+            results = self.spectral_smooth()
+
+        if add_data:
+            # add data to the collection/viewer
+            self.add_results.add_results_from_plugin(results)
+            self._set_default_results_label()
+
+        snackbar_message = SnackbarMessage(
+            f"Data set '{self.dataset_selected}' smoothed successfully.",
+            color="success",
+            sender=self)
+        self.hub.broadcast(snackbar_message)
+
+        return results
+
+    def spectral_smooth(self):
+        """
+        Smooth the input spectrum along the spectral axis.  To add the resulting spectrum into
+        the app, set label options and use :meth:`smooth` instead.
+
+        Returns
+        -------
+        spec : `~specutils.Spectrum1D`
+            The smoothed spectrum
+        """
         # Testing inputs to make sure putting smoothed spectrum into
         # datacollection works
         # input_flux = Quantity(np.array([0.2, 0.3, 2.2, 0.3]), u.Jy)
@@ -93,30 +173,19 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
         cube = self.dataset.get_object(cls=Spectrum1D, statistic=None)
         spec_smoothed = gaussian_smooth(cube, stddev=self.stddev)
 
-        # add data to the collection/viewer
-        self.add_results.add_results_from_plugin(spec_smoothed)
+        return spec_smoothed
 
-        snackbar_message = SnackbarMessage(
-            f"Data set '{self.dataset_selected}' smoothed successfully.",
-            color="success",
-            sender=self)
-        self.hub.broadcast(snackbar_message)
-
-    def apply_spatial_convolution(self):
+    def spatial_smooth(self):
         """
         Use astropy convolution machinery to smooth the spatial dimensions of
-        the data cube.
+        the data cube.  To add the resulting cube into
+        the app, set label options and use :meth:`smooth` instead.
+
+        Returns
+        -------
+        cube : `~specutils.Spectrum1D`
+            The smoothed cube
         """
-        if self.results_label in self.data_collection:
-            # immediately cancel before smoothing
-            snackbar_message = SnackbarMessage(
-                "Data with selected stddev already exists, canceling operation.",
-                color="error",
-                sender=self)
-            self.hub.broadcast(snackbar_message)
-
-            return
-
         # Get information from the flux component
         attribute = self.dataset.selected_dc_item.main_components[0]
 
@@ -129,25 +198,10 @@ class GaussianSmooth(PluginTemplateMixin, DatasetSelectMixin, AddResultsMixin):
         # we can do "3d" convolution to the whole cube
         kernel = np.expand_dims(Gaussian2DKernel(self.stddev), 2)
 
-        # TODO: in vuetify >2.3, timeout should be set to -1 to keep open
-        #  indefinitely
-        snackbar_message = SnackbarMessage(
-            "Smoothing spatial slices of cube...",
-            loading=True, timeout=0, sender=self)
-        self.hub.broadcast(snackbar_message)
-
         convolved_data = convolve(cube, kernel)
 
         # Create a new cube with the old metadata. Note that astropy
         # convolution generates values for masked (NaN) data.
         newcube = Spectrum1D(flux=convolved_data * flux_unit, wcs=cube.wcs)
 
-        # add data to the collection/plots
-        self.add_results.add_results_from_plugin(newcube)
-        self._set_default_results_label()
-
-        snackbar_message = SnackbarMessage(
-            f"Data set '{self.dataset_selected}' smoothed successfully.",
-            color="success",
-            sender=self)
-        self.hub.broadcast(snackbar_message)
+        return newcube
