@@ -10,12 +10,14 @@ from glue.core.data import Data
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin,
+                                        SelectPluginComponent,
                                         SpectralSubsetSelectMixin,
                                         SubsetSelect,
                                         DatasetSelectMixin,
-                                        AutoLabel,
+                                        AutoTextField,
                                         AddResultsMixin)
 from jdaviz.core.custom_traitlets import IntHandleEmpty
+from jdaviz.core.user_api import PluginUserApi
 from jdaviz.configs.default.plugins.model_fitting.fitting_backend import fit_model_to_spectrum
 from jdaviz.configs.default.plugins.model_fitting.initializers import (MODELS,
                                                                        initialize,
@@ -35,6 +37,34 @@ class _EmptyParam:
 @tray_registry('g-model-fitting', label="Model Fitting")
 class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                    SpectralSubsetSelectMixin, AddResultsMixin):
+    """
+    See the :ref:`Model Fitting Plugin Documentation <specviz-model-fitting>` for more details.
+
+    Only the following attributes and methods are available through the
+    :ref:`public plugin API <plugin-apis>`:
+
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * ``dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
+      Dataset to fit the model.
+    * ``spatial_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`):
+      Only exposed for Cubeviz.  Spatially collapsed spectrum to use to fit the model.
+    * ``spectral_subset`` (:class:`~jdaviz.core.template_mixin.SubsetSelect`)
+    * ``model_component`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
+    * ``poly_order``
+    * ``model_component_label`` (:class:`~jdaviz.core.template_mixin.AutoTextField`)
+    * :meth:`create_model_component`
+    * :meth:`remove_model_component`
+    * :meth:`model_components`
+    * :meth:`get_model_component`
+    * :meth:`set_model_component`
+    * ``equation`` (:class:`~jdaviz.core.template_mixin.AutoTextField`)
+    * ``add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
+    * ``cube_fit``
+      Only exposed for Cubeviz.  Whether to fit the model to the cube instead of to the
+      collapsed spectrum.
+    * :meth:`calculate_fit`
+    """
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "model_fitting.vue"
     form_valid_model_component = Bool(False).tag(sync=True)
@@ -43,8 +73,8 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     spatial_subset_selected = Unicode().tag(sync=True)
 
     # model components:
-    available_comps = List(list(MODELS.keys())).tag(sync=True)
-    comp_selected = Unicode().tag(sync=True)
+    model_comp_items = List().tag(sync=True)
+    model_comp_selected = Unicode().tag(sync=True)
     poly_order = IntHandleEmpty(0).tag(sync=True)
 
     comp_label = Unicode().tag(sync=True)
@@ -75,7 +105,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self._display_order = False
         self._window = None
         self._original_mask = None
-        if self.app.state.settings.get("configuration") == "cubeviz":
+        if self.config == "cubeviz":
             self.spatial_subset = SubsetSelect(self,
                                                'spatial_subset_items',
                                                'spatial_subset_selected',
@@ -84,19 +114,41 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         else:
             self.spatial_subset = None
 
+        # create the label first so that when model_component defaults to the first selection,
+        # the label automatically defaults as well
+        self.model_component_label = AutoTextField(self, 'comp_label', 'comp_label_default',
+                                                   'comp_label_auto', 'comp_label_invalid_msg')
+
+        self.model_component = SelectPluginComponent(self,
+                                                     items='model_comp_items',
+                                                     selected='model_comp_selected',
+                                                     manual_options=list(MODELS.keys()))
+
         # when accessing the selected data, access the spectrum-viewer version
         self.dataset._viewers = ['spectrum-viewer']
         # require entries to be in spectrum-viewer (not other cubeviz images, etc)
         self.dataset.add_filter('layer_in_spectrum_viewer')
 
-        self.auto_component_label = AutoLabel(self, 'comp_label', 'comp_label_default',
-                                              'comp_label_auto', 'comp_label_invalid_msg')
-        self.auto_equation = AutoLabel(self, 'model_equation', 'model_equation_default',
-                                       'model_equation_auto', 'model_equation_invalid_msg')
+        self.equation = AutoTextField(self, 'model_equation', 'model_equation_default',
+                                      'model_equation_auto', 'model_equation_invalid_msg')
 
         # set the filter on the viewer options
         self._update_viewer_filters()
 
+    @property
+    def user_api(self):
+        expose = ['dataset']
+        if self.config == "cubeviz":
+            expose += ['spatial_subset']
+        expose += ['spectral_subset', 'model_component', 'poly_order', 'model_component_label',
+                   'model_components', 'create_model_component', 'remove_model_component',
+                   'get_model_component', 'set_model_component',
+                   'equation', 'add_results']
+        if self.config == "cubeviz":
+            expose += ['cube_fit']
+        expose += ['calculate_fit']
+        return PluginUserApi(self, expose=expose)
+
     def _param_units(self, param, model_type=None):
         """Helper function to handle units that depend on x and y"""
         y_params = ["amplitude", "amplitude_L", "intercept", "scale"]
@@ -117,23 +169,36 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     def _update_parameters_from_fit(self):
         """Insert the results of the model fit into the component_models"""
         for m in self.component_models:
+            submodel_index = None
             name = m["id"]
             if hasattr(self._fitted_model, "submodel_names"):
-                if name in self._fitted_model.submodel_names:
-                    m_fit = self._fitted_model[name]
-                else:
+                for i in range(len(self._fitted_model.submodel_names)):
+                    if name == self._fitted_model.submodel_names[i]:
+                        m_fit = self._fitted_model[name]
+                        submodel_index = i
+                if submodel_index is None:
                     continue
             elif self._fitted_model.name == name:
                 m_fit = self._fitted_model
             else:
                 # then the component was not in the fitted model
                 continue
+
             temp_params = []
             for i in range(0, len(m_fit.parameters)):
                 temp_param = [x for x in m["parameters"] if x["name"] ==
                               m_fit.param_names[i]]
                 temp_param[0]["value"] = m_fit.parameters[i]
+                # The submodels don't have uncertainties attached, only the compound model
+                if self._fitted_model.stds is not None:
+                    std_name = temp_param[0]["name"]
+                    if submodel_index is not None:
+                        std_name = f"{std_name}_{submodel_index}"
+                    if std_name in self._fitted_model.stds.param_names:
+                        temp_param[0]["std"] = self._fitted_model.stds[std_name]
+
                 temp_params += temp_param
+
             m["parameters"] = temp_params
 
         # Trick traitlets into updating the displayed values
@@ -265,15 +330,11 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             spectral_min, spectral_max = self.spectral_subset.selected_min_max(self._spectrum1d)
             self._window = u.Quantity([spectral_min, spectral_max])
 
-    @observe('comp_selected', 'poly_order')
-    def _update_comp_label_default(self, event={}):
+    def _default_comp_label(self, model, poly_order=None):
         abbrevs = {'BlackBody': 'BB', 'PowerLaw': 'PL', 'Lorentz1D': 'Lo'}
-        abbrev = abbrevs.get(self.comp_selected, self.comp_selected[0].upper())
-        if self.comp_selected == "Polynomial1D":
-            self.display_order = True
-            abbrev += f'{self.poly_order}'
-        else:
-            self.display_order = False
+        abbrev = abbrevs.get(model, model[0].upper())
+        if model == "Polynomial1D":
+            abbrev += f'{poly_order}'
 
         # append a number suffix to avoid any duplicates
         ind = 1
@@ -281,7 +342,13 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             abbrev = f'{abbrev.split("_")[0]}_{ind}'
             ind += 1
 
-        self.comp_label_default = abbrev
+        return abbrev
+
+    @observe('model_comp_selected', 'poly_order')
+    def _update_comp_label_default(self, event={}):
+        self.display_order = self.model_comp_selected == "Polynomial1D"
+        self.comp_label_default = self._default_comp_label(self.model_comp_selected,
+                                                           self.poly_order)
 
     @observe('comp_label')
     def _comp_label_changed(self, event={}):
@@ -322,30 +389,62 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         return temp_models
 
-    def vue_add_model(self, event={}):
-        """Add the selected model and input string ID to the list of models"""
+    def create_model_component(self, model_component=None, model_component_label=None,
+                               poly_order=None):
+        """
+        Add a new model component to the list of available model components
+
+        Parameters
+        ----------
+        model_component : str
+            Type of model component to add.  If not provided, will default according to
+            ``model_component``.
+        model_component_label : str
+            Name of the model component to add.  If not provided, will default according to
+            ``model_component_label`` (if ``model_component_label.auto`` is True and
+            ``model_component`` is passed as an argument, then the default label will be recomputed
+            rather than applying the current value).
+        poly_order : int
+            Order of the polynomial if ``model_component`` is (or defaults to) "Polynomial1D".
+            Will raise an error if provided and ``model_component`` is not "Polynomial1D".
+            If not provided, will default according to ``poly_order``.
+        """
         if not self._spectrum1d:
             self._dataset_selected_changed()
 
+        model_comp = model_component if model_component is not None else self.model_comp_selected
+
+        if model_comp != "Polynomial1D" and poly_order is not None:
+            raise ValueError("poly_order should only be passed if model_component is Polynomial1D")
+        poly_order = poly_order if poly_order is not None else self.poly_order
+
+        # if model_component was passed and different than the one set in the traitlet, AND
+        # model_component_label is not passed, AND the auto is enabled on the label, then
+        # recompute a temporary default model label rather than use the value set in the traitlet
+        if model_comp != self.model_comp_selected and model_component_label is None and self.model_component_label.auto:  # noqa
+            comp_label = self._default_comp_label(model_comp, poly_order)
+        else:
+            comp_label = model_component_label if model_component_label is not None else self.comp_label  # noqa
+
         # validate provided label (only allow "word characters").   These should already be
         # stripped by JS in the UI element, but we'll confirm here (especially if this is ever
         # extended to have better API-support)
-        if re.search(r'\W+', self.comp_label):
-            raise ValueError(f"invalid model component ID {self.comp_label}")
+        if re.search(r'\W+', comp_label) or not len(comp_label):
+            raise ValueError(f"invalid model component label '{comp_label}'")
 
-        if self.comp_label in [cm['id'] for cm in self.component_models]:
-            raise ValueError(f"model component ID {self.comp_label} already in use")
+        if comp_label in [cm['id'] for cm in self.component_models]:
+            raise ValueError(f"model component label '{comp_label}' already in use")
 
-        new_model = {"id": self.comp_label, "model_type": self.comp_selected,
+        new_model = {"id": comp_label, "model_type": model_comp,
                      "parameters": [], "model_kwargs": {}}
-        model_cls = MODELS[self.comp_selected]
+        model_cls = MODELS[model_comp]
 
-        if self.comp_selected == "Polynomial1D":
+        if model_comp == "Polynomial1D":
             # self.poly_order is the value in the widget for creating
             # the new model component.  We need to store that with the
             # model itself as the value could change for another component.
-            new_model["model_kwargs"] = {"degree": self.poly_order}
-        elif self.comp_selected == "BlackBody":
+            new_model["model_kwargs"] = {"degree": poly_order}
+        elif model_comp == "BlackBody":
             new_model["model_kwargs"] = {"output_units": self._units["y"],
                                          "bounds": {"scale": (0.0, None)}}
 
@@ -369,9 +468,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             initial_values[param_name] = initial_val
 
         initialized_model = initialize(
-            MODELS[self.comp_selected](name=self.comp_label,
-                                       **initial_values,
-                                       **new_model.get("model_kwargs", {})),
+            MODELS[model_comp](name=comp_label,
+                               **initial_values,
+                               **new_model.get("model_kwargs", {})),
             self._spectrum1d.spectral_axis,
             self._spectrum1d.flux)
 
@@ -384,7 +483,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                                             "unit": str(param_quant.unit),
                                             "fixed": False})
 
-        self._initialized_models[self.comp_label] = initialized_model
+        self._initialized_models[comp_label] = initialized_model
 
         new_model["Initialized"] = True
         self.component_models = self.component_models + [new_model]
@@ -392,22 +491,115 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self._update_comp_label_default()
         self._update_model_equation_default()
 
-    def vue_remove_model(self, event):
+    def remove_model_component(self, model_component_label):
+        """
+        Remove an existing model component.
+
+        Parameters
+        ----------
+        model_component_label : str
+            The label given to the existing model component
+        """
+        if model_component_label not in [x["id"] for x in self.component_models]:
+            raise ValueError(f"model component with label '{model_component_label}' does not exist")
+
         self.component_models = [x for x in self.component_models
-                                 if x["id"] != event]
-        del self._initialized_models[event]
+                                 if x["id"] != model_component_label]
+        del self._initialized_models[model_component_label]
         self._update_comp_label_default()
         self._update_model_equation_default()
 
+    def get_model_component(self, model_component_label, parameter=None):
+        """
+        Get a (read-only) dictionary representation of an existing model component.
+
+        Parameters
+        ----------
+        model_component_label : str
+            The label given to the existing model component
+        parameter : str
+            Optional.  The name of a valid parameter in the model component, in which case only
+            the information on that parameter is returned.
+        """
+        try:
+            model_component = [x for x in self.component_models
+                               if x["id"] == model_component_label][0]
+        except IndexError:
+            raise ValueError(f"'{model_component_label}' is not a label of an existing model component")  # noqa
+
+        comp = {"model_type": model_component['model_type'],
+                "parameters": {p['name']: {'value': p['value'],
+                                           'unit': p['unit'],
+                                           'fixed': p['fixed']} for p in model_component['parameters']}}  # noqa
+
+        if parameter is not None:
+            return comp['parameters'].get(parameter)
+        return comp
+
+    def set_model_component(self, model_component_label, parameter, value=None, fixed=None):
+        """
+        Set the value or fixed attribute of a parameter in an existing model component.
+
+        Parameters
+        ----------
+        model_component_label : str
+            The label given to the existing model component
+        parameter : str
+            The name of a valid parameter in the model component.
+        value : float
+            Optional.  The new initial value of the parameter.  If not provided or None, will
+            remain unchanged.
+        fixed : bool
+            Optional.  The new state of the fixed attribute of the parameter.  If not provided
+            or None, will remain unchanged.
+
+        Returns
+        -------
+        updated dictionary of the parameter representation
+        """
+        cms = self.component_models
+        try:
+            model_component = [x for x in cms if x["id"] == model_component_label][0]
+        except IndexError:
+            raise ValueError(f"'{model_component_label}' is not a label of an existing model component")  # noqa
+        try:
+            parameter = [p for p in model_component['parameters'] if p['name'] == parameter][0]
+        except IndexError:
+            raise ValueError(f"'{parameter}' is not the name of a parameter in the '{model_component_label}' model component")  # noqa
+
+        if value is not None:
+            if not isinstance(value, (int, float)):
+                raise TypeError("value must be a float")
+            parameter['value'] = value
+        if fixed is not None:
+            if not isinstance(fixed, bool):
+                raise TypeError("fixed must be a boolean")
+            parameter['fixed'] = fixed
+
+        self.component_models = []
+        self.component_models = cms
+
+        return parameter
+
+    @property
+    def model_components(self):
+        """
+        List of the labels of existing model components
+        """
+        return [x["id"] for x in self.component_models]
+
+    def vue_add_model(self, event):
+        self.create_model_component()
+
+    def vue_remove_model(self, event):
+        self.remove_model_component(event)
+
     @observe('model_equation')
     def _model_equation_changed(self, event):
         # Length is a dummy check to test the infrastructure
         if len(self.model_equation) == 0:
             self.model_equation_invalid_msg = 'model equation is required'
             return
-        if len(self.model_equation) > 20:
-            self.model_equation_invalid_msg = 'model equation too long'
-            return
         self.model_equation_invalid_msg = ''
 
     @observe("dataset_selected", "dataset_items", "cube_fit")
@@ -429,13 +621,30 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             # only want spectral viewers in the options
             self.add_results.viewer.filters = ['is_spectrum_viewer']
 
-    def vue_apply(self, event):
-        if self.cube_fit:
-            self.vue_fit_model_to_cube()
-        else:
-            self.vue_model_fitting()
+    def calculate_fit(self, add_data=True):
+        """
+        Calculate the fit.
 
-    def vue_model_fitting(self, *args, **kwargs):
+        Parameters
+        ----------
+        add_data : bool
+            Whether to add the resulting spectrum/cube to the app as a data entry according to
+            ``add_results``.
+
+        Returns
+        -------
+        fitted model
+        fitted spectrum/cube
+        """
+        if self.cube_fit:
+            return self._fit_model_to_cube(add_data=add_data)
+        else:
+            return self._fit_model_to_spectrum(add_data=add_data)
+
+    def vue_apply(self, event):
+        self.calculate_fit()
+
+    def _fit_model_to_spectrum(self, add_data):
         """
         Run fitting on the initialized models, fixing any parameters marked
         as such by the user, then update the displayed parameters with fit
@@ -465,14 +674,16 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
                                   color="error", sender=self)
             self.hub.broadcast(msg)
             return
+
         self._fitted_model = fitted_model
         self._fitted_spectrum = fitted_spectrum
 
-        self.app.fitted_models[self.results_label] = fitted_model
-        self.vue_register_spectrum({"spectrum": fitted_spectrum})
+        if add_data:
+            self.app.fitted_models[self.results_label] = fitted_model
+            self._register_spectrum({"spectrum": fitted_spectrum})
 
         # Update component model parameters with fitted values
-        if type(self._fitted_model) == QuantityModel:
+        if isinstance(self._fitted_model, QuantityModel):
             self._update_parameters_from_QM()
         else:
             self._update_parameters_from_fit()
@@ -484,7 +695,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # Reset the data mask in case we use a different subset next time
         self._spectrum1d.mask = self._original_mask
 
-    def vue_fit_model_to_cube(self, *args, **kwargs):
+        return fitted_model, fitted_spectrum
+
+    def _fit_model_to_cube(self, add_data):
 
         if self._warn_if_no_equation():
             return
@@ -531,30 +744,32 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         # Save fitted 3D model in a way that the cubeviz
         # helper can access it.
-        for m in fitted_model:
-            temp_label = "{} ({}, {})".format(self.results_label, m["x"], m["y"])
-            self.app.fitted_models[temp_label] = m["model"]
+        if add_data:
+            for m in fitted_model:
+                temp_label = "{} ({}, {})".format(self.results_label, m["x"], m["y"])
+                self.app.fitted_models[temp_label] = m["model"]
 
         count = max(map(lambda s: int(next(iter(re.findall(r"\d$", s)), 0)),
                         self.data_collection.labels)) + 1
 
-        label = f"{self.results_label} [Cube] {count}"
-
         # Create new glue data object
-        output_cube = Data(label=label,
+        output_cube = Data(label=f"{self.results_label} [Cube] {count}",
                            coords=data.coords)
         output_cube['flux'] = fitted_spectrum.flux.value
         output_cube.get_component('flux').units = fitted_spectrum.flux.unit.to_string()
 
-        self.add_results.add_results_from_plugin(output_cube)
-        self._set_default_results_label()
+        if add_data:
+            self.add_results.add_results_from_plugin(output_cube)
+            self._set_default_results_label()
 
         snackbar_message = SnackbarMessage(
             "Finished cube fitting",
             color='success', loading=False, sender=self)
         self.hub.broadcast(snackbar_message)
 
-    def vue_register_spectrum(self, event):
+        return fitted_model, output_cube
+
+    def _register_spectrum(self, event):
         """
         Add a spectrum to the data collection based on the currently displayed
         parameters (these could be user input or fit values).
