@@ -255,15 +255,28 @@ def mos_spec2d_parser(app, data_obj, data_labels=None, add_to_table=True,
     data_labels : str, optional
         The label applied to the glue data component.
     """
+    # Note: This is also used by Specviz2D
     def _parse_as_spectrum1d(path):
         # Parse as a FITS file and assume the WCS is correct
         with fits.open(path) as hdulist:
             data = hdulist[1].data
             header = hdulist[1].header
-            wcs = WCS(header)
             metadata = standardize_metadata(header)
             metadata[PRIHDR_KEY] = standardize_metadata(hdulist[0].header)
-        return Spectrum1D(data, wcs=wcs, meta=metadata)
+            wcs = WCS(header)
+
+            try:
+                data_unit = u.Unit(header['BUNIT'])
+            except Exception:
+                data_unit = u.count
+
+            # FITS WCS is invalid, so ignore it.
+            if wcs.spectral.naxis == 0:
+                kw = {}
+            else:
+                kw = {'wcs': wcs}
+
+        return Spectrum1D(flux=data * data_unit, meta=metadata, **kw)
 
     # Coerce into list-like object
     if not isinstance(data_obj, (list, tuple, SpectrumCollection)):
@@ -708,7 +721,15 @@ def mos_niriss_parser(app, data_dir):
                         # The wavelength is stored in a WAVELENGTH HDU. This is
                         # a 2D array, but in order to be able to use Spectrum1D
                         # we use the average wavelength for all image rows
-                        wav = temp[wav_hdus[sci]].data.mean(axis=0) * u.micron
+
+                        if data.shape[0] > data.shape[1]:
+                            # then the input data needs to be transposed, and wavelength
+                            # needs to be averaged over axis=1 instead of axis=0
+                            data = data.T
+                            wav = temp[wav_hdus[sci]].data.mean(axis=1) * u.micron
+                        else:
+                            wav = temp[wav_hdus[sci]].data.mean(axis=0) * u.micron
+
                         spec2d = Spectrum1D(data * u.one, spectral_axis=wav, meta=meta)
                         spec2d.meta['INSTRUME'] = 'NIRISS'
                         spec2d.meta['mosviz_row'] = len(spec_labels_2d)
@@ -734,21 +755,25 @@ def mos_niriss_parser(app, data_dir):
             print(f"Loading: {flabel} sources")
 
             with fits.open(fname, memmap=False) as temp:
-                # treat all HDUs missing SRCTYPE attribute as extended sources
-                # TODO: Remove this once valid SRCTYPE values are present in all headers
-                for hdu in temp:
-                    if ("SRCTYPE" in hdu.header
-                            and (hdu.header["SRCTYPE"] in ("POINT", "EXTENDED"))):
-                        pass
-                    else:
-                        hdu.header["SRCTYPE"] = "EXTENDED"
+                # Filter out HDUs we care about
+                source_ids_to_filter = cat_id_dict.keys()
+                filtered_hdul = fits.HDUList([hdu for hdu in temp if (
+                    (hdu.name in ('PRIMARY', 'ASDF')) or
+                    (hdu.header.get('SOURCEID', None) in source_ids_to_filter))])
 
-                # read all HDUs with SpectrumList, then only keep those that
-                # correspond with sources in catalog
-                # (read() is slow... would also LOVE to do this in one step!)
-                specs = SpectrumList.read(temp, format="JWST x1d multi")
-                specs_cut = [sp for sp in specs if sp.meta['header']['SOURCEID']
-                             in cat_id_dict.keys()]
+                # SRCTYPE is required for the specutils JWST x1d reader. The reader will
+                # force this to POINT if not set. Under known cases, this field will be set
+                # for NIRISS programs; if it's not, something's gone wrong. Catch this
+                # warning and reraise as an error to warn users.
+                try:
+                    with warnings.catch_warnings():
+                        warnings.filterwarnings("error",
+                                                category=UserWarning,
+                                                message=".*SRCTYPE is missing or UNKNOWN*")
+                        specs = SpectrumList.read(filtered_hdul, format="JWST x1d multi")
+                except UserWarning as e:
+                    raise KeyError(f"The SRCTYPE keyword in the header of file {fname}"
+                                   "is not populated (expected values: EXTENDED or POINT)") from e
 
                 filter_name = fits.getheader(fname, ext=0).get('PUPIL')
 
@@ -756,7 +781,7 @@ def mos_niriss_parser(app, data_dir):
                 orientation = flabel.split()[-1]
 
                 # update 1D labels and standardize metadata for table viewer
-                for sp in specs_cut:
+                for sp in specs:
                     if (
                         sp.meta['header']['SPORDER'] == 1
                         and sp.meta['header']['EXTNAME'] == 'EXTRACT1D'
