@@ -193,26 +193,6 @@ def test_nirspec_loader(mosviz_helper, tmpdir):
     assert dc_15.meta['SOURCEID'] == 2315
 
 
-# This is another version of test_niriss_parser in test_parsers.py
-@pytest.mark.remote_data
-def test_niriss_loader(mosviz_helper, tmpdir):
-
-    test_data = 'https://stsci.box.com/shared/static/l2azhcqd3tvzhybdlpx2j2qlutkaro3z.zip'
-    fn = download_file(test_data, cache=True, timeout=30)
-    with ZipFile(fn, 'r') as sample_data_zip:
-        sample_data_zip.extractall(tmpdir)
-
-    level3_path = (pathlib.Path(tmpdir) / 'NIRISS_for_parser_p0171')
-
-    data_dir = level3_path
-
-    mosviz_helper.load_data(directory=data_dir, instrument='niriss')
-
-    assert len(mosviz_helper.app.data_collection) == 80
-    assert mosviz_helper.app.data_collection[0].label == "Image canucs F150W"
-    assert mosviz_helper.app.data_collection[-1].label == "MOS Table"
-
-
 @pytest.mark.remote_data
 def test_nirspec_fallback(mosviz_helper, tmpdir):
     '''
