@@ -265,9 +265,32 @@ def test_plot_uncertainties(specviz_helper, spectrum1d):
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
 
     specviz_viewer.state.show_uncertainty = True
+    uncert_marks = [m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]
+    assert len(uncert_marks) == 1
+    # mark has a lower and upper boundary, but each should have the same length as the spectrum
+    assert len(uncert_marks[0].x[0]) == len(spectrum1d.flux)
 
-    assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 1
+    # enable as-steps and make sure doesn't crash (won't change the number of marks)
+    specviz_viewer.state.layers[0].as_steps = True
+    specviz_viewer._plot_uncertainties()
+    uncert_marks = [m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]
+    assert len(uncert_marks) == 1
+    # now the mark should have double the length as its drawn on the bin edges
+    assert len(uncert_marks[0].x[0]) == len(spectrum1d.flux) * 2
 
     specviz_viewer.state.show_uncertainty = False
 
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
+
+
+def test_plugin_user_apis(specviz_helper):
+    for plugin_name, plugin_api in specviz_helper.plugins.items():
+        plugin = plugin_api._obj
+        for attr in plugin_api._expose:
+            assert hasattr(plugin, attr)
+
+
+def test_data_label_as_posarg(specviz_helper, spectrum1d):
+    # Passing in data_label keyword as posarg.
+    specviz_helper.load_data(spectrum1d, 'my_spec')
+    assert specviz_helper.app.data_collection[0].label == 'my_spec'
