@@ -6,11 +6,14 @@ from traitlets import Bool, List, Unicode, observe
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.core.registries import tray_registry
 from jdaviz.core.template_mixin import (PluginTemplateMixin,
+                                        SelectPluginComponent,
                                         DatasetSelect,
                                         AddResults)
-from jdaviz.core.custom_traitlets import IntHandleEmpty
+from jdaviz.core.user_api import PluginUserApi
+from jdaviz.core.custom_traitlets import IntHandleEmpty, FloatHandleEmpty
 from jdaviz.core.marks import PluginLine
 
+from astropy.nddata import NDData, StdDevUncertainty, VarianceUncertainty, UnknownUncertainty
 from specutils import Spectrum1D
 from specreduce import tracing
 from specreduce import background
@@ -21,38 +24,111 @@ __all__ = ['SpectralExtraction']
 
 @tray_registry('spectral-extraction', label="Spectral Extraction")
 class SpectralExtraction(PluginTemplateMixin):
+    """
+    The Spectral Extraction plugin exposes specreduce methods for tracing, background subtraction,
+    and spectral extraction from 2D spectra.
+    See the :ref:`Spectral Extraction Plugin Documentation <specviz2d-spectral-extraction>`
+    for more details.
+
+    Only the following attributes and methods are available through the
+    :ref:`public plugin API <plugin-apis>`:
+
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.show`
+    * :meth:`~jdaviz.core.template_mixin.PluginTemplateMixin.open_in_tray`
+    * :attr:`interactive_extract`
+    * ``trace_dataset`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      controls the input dataset for generating the trace.
+    * ``trace_type`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      controls the type of trace to be generated.
+    * ``trace_peak_method`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      only applicable if ``trace_type`` is set to ``Auto``.
+    * :attr:`trace_pixel` :
+      pixel of the trace.  If ``trace_type`` is set to ``Auto``, then this
+      is the "guess" for the automated trace.
+    * :attr:`trace_bins` :
+      only applicable if ``trace_type`` is set to ``Auto``.
+    * :attr:`trace_window` :
+      full width of the trace.
+    * :meth:`import_trace`
+    * :meth:`export_trace`
+    * ``bg_dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
+      controls the input dataset for generating the background.
+    * ``bg_type`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`):
+      controls the type of background to be generated.
+    * :attr:`bg_trace_pixel` :
+      only applicable if ``bg_type`` is set to ``Manual``
+    * :attr:`bg_separation` :
+      only applicable if ``bg_type`` set set to ``OneSided`` or ``TwoSided``.
+      Separation from the referenced trace for the center of each of the background window(s).
+    * :attr:`bg_width` :
+      full width of each background window(s).
+    * ``bg_add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
+    * ``bg_sub_add_results``
+    * :meth:`import_bg`
+    * :meth:`export_bg`
+    * :meth:`export_bg_img`
+    * :meth:`export_bg_sub`
+    * ``ext_dataset`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`):
+      controls the input dataset for generating the extracted spectrum.  "From Plugin" will
+      use the background-subtracted image as defined by the background options above.  To skip
+      background extraction, select the original input 2D spectrum.
+    * ``ext_trace`` (:class:`~jdaviz.core.template_mixin.DatasetSelect`)
+    * ``ext_type`` (:class:`~jdaviz.core.template_mixin.SelectPluginComponent`)
+    * :attr:`ext_width` :
+      full width of the extraction window.
+    * ``ext_add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
+    * :meth:`import_extract`
+    * :meth:`export_extract`
+    * :meth:`export_extract_spectrum`
+    """
     dialog = Bool(False).tag(sync=True)
     template_file = __file__, "spectral_extraction.vue"
 
     active_step = Unicode().tag(sync=True)
-    setting_interactive_extract = Bool(True).tag(sync=True)
+    interactive_extract = Bool(True).tag(sync=True)
 
     # TRACE
+    trace_trace_items = List().tag(sync=True)
+    trace_trace_selected = Unicode().tag(sync=True)
+
+    trace_offset = IntHandleEmpty(0).tag(sync=True)
+
     trace_dataset_items = List().tag(sync=True)
     trace_dataset_selected = Unicode().tag(sync=True)
 
-    trace_type_items = List(['Flat', 'Auto']).tag(sync=True)
-    trace_type_selected = Unicode('Flat').tag(sync=True)
+    trace_type_items = List().tag(sync=True)
+    trace_type_selected = Unicode().tag(sync=True)
 
-    trace_pixel = IntHandleEmpty(0).tag(sync=True)
+    trace_pixel = FloatHandleEmpty(0).tag(sync=True)
 
-    trace_peak_method_items = List(['Gaussian', 'Centroid', 'Max']).tag(sync=True)
-    trace_peak_method_selected = Unicode('Gaussian').tag(sync=True)
+    trace_peak_method_items = List().tag(sync=True)
+    trace_peak_method_selected = Unicode().tag(sync=True)
 
     trace_bins = IntHandleEmpty(20).tag(sync=True)
     trace_window = IntHandleEmpty(0).tag(sync=True)
 
+    trace_results_label = Unicode().tag(sync=True)
+    trace_results_label_default = Unicode().tag(sync=True)
+    trace_results_label_auto = Bool(True).tag(sync=True)
+    trace_results_label_invalid_msg = Unicode('').tag(sync=True)
+    trace_results_label_overwrite = Bool().tag(sync=True)
+    trace_add_to_viewer_items = List().tag(sync=True)
+    trace_add_to_viewer_selected = Unicode().tag(sync=True)
+
     # BACKGROUND
     bg_dataset_items = List().tag(sync=True)
     bg_dataset_selected = Unicode().tag(sync=True)
 
-    bg_type_items = List(['TwoSided', 'OneSided', 'Manual']).tag(sync=True)
-    bg_type_selected = Unicode('TwoSided').tag(sync=True)
+    bg_type_items = List().tag(sync=True)
+    bg_type_selected = Unicode().tag(sync=True)
 
-    bg_trace_pixel = IntHandleEmpty(0).tag(sync=True)
+    bg_trace_items = List().tag(sync=True)
+    bg_trace_selected = Unicode().tag(sync=True)
 
-    bg_separation = IntHandleEmpty(0).tag(sync=True)
-    bg_width = IntHandleEmpty(0).tag(sync=True)
+    bg_trace_pixel = FloatHandleEmpty(0).tag(sync=True)
+
+    bg_separation = FloatHandleEmpty(0).tag(sync=True)
+    bg_width = FloatHandleEmpty(0).tag(sync=True)
 
     bg_results_label = Unicode().tag(sync=True)
     bg_results_label_default = Unicode().tag(sync=True)
@@ -74,8 +150,15 @@ class SpectralExtraction(PluginTemplateMixin):
     ext_dataset_items = List().tag(sync=True)
     ext_dataset_selected = Unicode().tag(sync=True)
 
-    ext_width = IntHandleEmpty(0).tag(sync=True)
+    ext_trace_items = List().tag(sync=True)
+    ext_trace_selected = Unicode().tag(sync=True)
 
+    ext_type_items = List().tag(sync=True)
+    ext_type_selected = Unicode().tag(sync=True)
+
+    ext_width = FloatHandleEmpty(0).tag(sync=True)
+
+    ext_uncert_warn = Bool(False).tag(sync=True)
     ext_specreduce_err = Unicode().tag(sync=True)
 
     ext_results_label = Unicode().tag(sync=True)
@@ -92,17 +175,54 @@ class SpectralExtraction(PluginTemplateMixin):
         self._do_marks = kwargs.get('interactive', True)
 
         # TRACE
+        self.trace_trace = DatasetSelect(self,
+                                         'trace_trace_items',
+                                         'trace_trace_selected',
+                                         default_text='New Trace',
+                                         filters=['is_trace'])
+
         self.trace_dataset = DatasetSelect(self,
                                            'trace_dataset_items',
                                            'trace_dataset_selected',
                                            filters=['layer_in_spectrum_2d_viewer', 'not_trace'])
 
+        self.trace_type = SelectPluginComponent(self,
+                                                items='trace_type_items',
+                                                selected='trace_type_selected',
+                                                manual_options=['Flat', 'Auto'])
+
+        self.trace_peak_method = SelectPluginComponent(self,
+                                                       items='trace_peak_method_items',
+                                                       selected='trace_peak_method_selected',
+                                                       manual_options=['Gaussian', 'Centroid' 'Max'])  # noqa
+
+        self.trace_add_results = AddResults(self, 'trace_results_label',
+                                            'trace_results_label_default',
+                                            'trace_results_label_auto',
+                                            'trace_results_label_invalid_msg',
+                                            'trace_results_label_overwrite',
+                                            'trace_add_to_viewer_items',
+                                            'trace_add_to_viewer_selected')
+        self.trace_add_results.viewer.filters = ['is_spectrum_2d_viewer']
+        self.trace_results_label_default = 'trace'
+
         # BACKGROUND
         self.bg_dataset = DatasetSelect(self,
                                         'bg_dataset_items',
                                         'bg_dataset_selected',
                                         filters=['layer_in_spectrum_2d_viewer', 'not_trace'])
 
+        self.bg_type = SelectPluginComponent(self,
+                                             items='bg_type_items',
+                                             selected='bg_type_selected',
+                                             manual_options=['TwoSided', 'OneSided', 'Manual'])
+
+        self.bg_trace = DatasetSelect(self,
+                                      items='bg_trace_items',
+                                      selected='bg_trace_selected',
+                                      default_text='From Plugin',
+                                      filters=['is_trace'])
+
         self.bg_add_results = AddResults(self, 'bg_results_label',
                                          'bg_results_label_default',
                                          'bg_results_label_auto',
@@ -130,6 +250,17 @@ class SpectralExtraction(PluginTemplateMixin):
                                          default_text='From Plugin',
                                          filters=['layer_in_spectrum_2d_viewer', 'not_trace'])
 
+        self.ext_trace = DatasetSelect(self,
+                                       items='ext_trace_items',
+                                       selected='ext_trace_selected',
+                                       default_text='From Plugin',
+                                       filters=['is_trace'])
+
+        self.ext_type = SelectPluginComponent(self,
+                                              items='ext_type_items',
+                                              selected='ext_type_selected',
+                                              manual_options=['Boxcar', 'Horne'])
+
         self.ext_add_results = AddResults(self, 'ext_results_label',
                                           'ext_results_label_default',
                                           'ext_results_label_auto',
@@ -142,6 +273,23 @@ class SpectralExtraction(PluginTemplateMixin):
         self.ext_add_results.label_whitelist_overwrite = ['Spectrum 1D']
         self.ext_results_label_default = 'Spectrum 1D'
 
+    @property
+    def user_api(self):
+        return PluginUserApi(self, expose=('interactive_extract',
+                                           'trace_dataset', 'trace_type', 'trace_peak_method',
+                                           'trace_pixel', 'trace_bins', 'trace_window',
+                                           'import_trace',
+                                           'export_trace',
+                                           'bg_dataset', 'bg_type',
+                                           'bg_trace_pixel', 'bg_separation', 'bg_width',
+                                           'bg_add_results', 'bg_sub_add_results',
+                                           'import_bg',
+                                           'export_bg', 'export_bg_img', 'export_bg_sub',
+                                           'ext_dataset', 'ext_trace', 'ext_type',
+                                           'ext_width', 'ext_add_results',
+                                           'import_extract',
+                                           'export_extract', 'export_extract_spectrum'))
+
     @observe('trace_dataset_selected')
     def _trace_dataset_selected(self, msg=None):
         if not hasattr(self, 'trace_dataset'):
@@ -169,7 +317,7 @@ class SpectralExtraction(PluginTemplateMixin):
         if self.bg_trace_pixel == 0:
             self.bg_trace_pixel = brightest_pixel
         if self.bg_separation == 0:
-            if default_bg_width * 2 > distance_from_edge:
+            if default_bg_width * 2 >= distance_from_edge:
                 self.bg_type_selected = 'OneSided'
                 # we want positive separation if brightest_pixel near bottom
                 sign = 1 if (brightest_pixel < width / 2) else -1
@@ -208,7 +356,7 @@ class SpectralExtraction(PluginTemplateMixin):
         """
         self.plugin_opened = False
 
-    @observe('plugin_opened', 'setting_interactive_extract')
+    @observe('plugin_opened', 'interactive_extract')
     def _update_plugin_marks(self, *args):
         if not self._do_marks:
             return
@@ -224,7 +372,7 @@ class SpectralExtraction(PluginTemplateMixin):
             return
 
         # update extracted 1d spectrum preview, regardless of the step
-        if self.setting_interactive_extract:
+        if self.interactive_extract:
             try:
                 sp1d = self.export_extract_spectrum(add_data=False)
             except Exception as e:
@@ -294,6 +442,7 @@ class SpectralExtraction(PluginTemplateMixin):
         return self._marks
 
     @observe('trace_dataset_selected', 'trace_type_selected',
+             'trace_trace_selected', 'trace_offset',
              'trace_pixel', 'trace_peak_method_selected',
              'trace_bins', 'trace_window', 'active_step')
     def _interaction_in_trace_step(self, event={}):
@@ -303,7 +452,7 @@ class SpectralExtraction(PluginTemplateMixin):
             return
 
         try:
-            trace = self.export_trace()
+            trace = self.export_trace(add_data=False)
         except Exception:
             # NOTE: ignore error, but will be raised when clicking ANY of the export buttons
             self.marks['trace'].clear()
@@ -314,7 +463,8 @@ class SpectralExtraction(PluginTemplateMixin):
         self.active_step = 'trace'
         self._update_plugin_marks()
 
-    @observe('bg_dataset_selected', 'bg_type_selected', 'bg_trace_pixel',
+    @observe('bg_dataset_selected', 'bg_type_selected',
+             'bg_trace_selected', 'bg_trace_pixel',
              'bg_separation', 'bg_width', 'active_step')
     def _interaction_in_bg_step(self, event={}):
         if not self.plugin_opened or not self._do_marks:
@@ -363,7 +513,8 @@ class SpectralExtraction(PluginTemplateMixin):
         self.active_step = 'bg'
         self._update_plugin_marks()
 
-    @observe('ext_dataset_selected', 'ext_width', 'active_step')
+    @observe('ext_dataset_selected', 'ext_trace_selected',
+             'ext_type_selected', 'ext_width', 'active_step')
     def _interaction_in_ext_step(self, event={}):
         if not self.plugin_opened or not self._do_marks:
             return
@@ -381,14 +532,26 @@ class SpectralExtraction(PluginTemplateMixin):
             self.marks['trace'].update_xy(xs,
                                           trace.trace)
             self.marks['trace'].line_style = 'dashed'
-            self.marks['ext_lower'].update_xy(xs,
-                                              trace.trace-self.ext_width/2)
-            self.marks['ext_upper'].update_xy(xs,
-                                              trace.trace+self.ext_width/2)
+            if self.ext_type_selected == 'Boxcar':
+                self.marks['ext_lower'].update_xy(xs,
+                                                  trace.trace-self.ext_width/2)
+                self.marks['ext_upper'].update_xy(xs,
+                                                  trace.trace+self.ext_width/2)
+            else:
+                for mark in ['ext_lower', 'ext_upper']:
+                    self.marks[mark].clear()
 
         self.active_step = 'ext'
         self._update_plugin_marks()
 
+        # TODO: remove this, the traitlet, and the row in spectral_extraction.vue
+        # when specutils handles the warning/exception
+        if self.ext_type_selected == 'Horne':
+            inp_sp2d = self._get_ext_input_spectrum()
+            self.ext_uncert_warn = isinstance(inp_sp2d.uncertainty, UnknownUncertainty)
+        else:
+            self.ext_uncert_warn = False
+
     def _set_create_kwargs(self, **kwargs):
         invalid_kwargs = [k for k in kwargs.keys() if not hasattr(self, k)]
         if len(invalid_kwargs):
@@ -417,19 +580,39 @@ class SpectralExtraction(PluginTemplateMixin):
             self.trace_pixel = trace.guess
             self.trace_window = trace.window
             self.trace_bins = trace.bins
+        elif isinstance(trace, tracing.ArrayTrace):  # pragma: no cover
+            raise NotImplementedError(f"cannot import ArrayTrace into plugin.  Use viz.load_trace instead")  # noqa
         else:  # pragma: no cover
             raise NotImplementedError(f"trace of type {trace.__class__.__name__} not supported")
 
-    def export_trace(self, **kwargs):
+    def export_trace(self, add_data=False, **kwargs):
         """
         Create a specreduce Trace object from the input parameters
         defined in the plugin.
+
+        Parameters
+        ----------
+        add_data : bool
+            Whether to add the resulting trace to the application, according to the options
+            defined in the plugin.
         """
         self._set_create_kwargs(**kwargs)
         if len(kwargs) and self.active_step != 'trace':
             self.update_marks(step='trace')
 
-        if self.trace_type_selected == 'Flat':
+        if self.trace_trace_selected != 'New Trace':
+            # then we're offsetting an existing trace
+            # for FlatTrace, we can keep and expose a new FlatTrace (which has the advantage of
+            # being able to load back into the plugin)
+            orig_trace = self.trace_trace.selected_obj
+            if isinstance(orig_trace, tracing.FlatTrace):
+                trace = tracing.FlatTrace(self.trace_dataset.selected_obj.data,
+                                          orig_trace.trace_pos+self.trace_offset)
+            else:
+                trace = tracing.ArrayTrace(self.trace_dataset.selected_obj.data,
+                                           self.trace_trace.selected_obj.trace+self.trace_offset)
+
+        elif self.trace_type_selected == 'Flat':
             trace = tracing.FlatTrace(self.trace_dataset.selected_obj.data,
                                       self.trace_pixel)
 
@@ -443,14 +626,22 @@ class SpectralExtraction(PluginTemplateMixin):
         else:
             raise NotImplementedError(f"trace_type={self.trace_type_selected} not implemented")
 
+        if add_data:
+            self.trace_add_results.add_results_from_plugin(trace, replace=False)
+
         return trace
 
+    def vue_create_trace(self, *args):
+        self.export_trace(add_data=True)
+
     def _get_bg_trace(self):
         if self.bg_type_selected == 'Manual':
             trace = tracing.FlatTrace(self.trace_dataset.selected_obj.data,
                                       self.bg_trace_pixel)
+        elif self.bg_trace_selected == 'From Plugin':
+            trace = self.export_trace(add_data=False)
         else:
-            trace = self.export_trace()
+            trace = self.bg_trace.selected_obj
 
         return trace
 
@@ -577,7 +768,10 @@ class SpectralExtraction(PluginTemplateMixin):
         self.export_bg_sub(add_data=True)
 
     def _get_ext_trace(self):
-        return self.export_trace()
+        if self.ext_trace_selected == 'From Plugin':
+            return self.export_trace(add_data=False)
+        else:
+            return self.ext_trace.selected_obj
 
     def _get_ext_input_spectrum(self):
         if self.ext_dataset_selected == 'From Plugin':
@@ -594,12 +788,13 @@ class SpectralExtraction(PluginTemplateMixin):
         ext : specreduce.extract.BoxcarExtract
             Extract object to import
         """
-        if not isinstance(ext, extract.BoxcarExtract):  # pragma: no cover
-            # TODO: add support for Optimal/Horne
-            raise TypeError("ext must be a specreduce.extract.BoxcarExtract object")
-
-        # TODO: should we detect/set the referenced dataset?
-        self.ext_width = ext.width
+        if isinstance(ext, extract.BoxcarExtract):
+            self.ext_type_selected = 'Boxcar'
+            self.ext_width = ext.width
+        elif isinstance(ext, extract.HorneExtract):
+            self.ext_type_selected = 'Horne'
+        else:  # pragma: no cover
+            raise TypeError("ext must be a specreduce.extract.BoxcarExtract or specreduce.extract.HorneExtract object")  # noqa
 
     def export_extract(self, **kwargs):
         """
@@ -610,9 +805,20 @@ class SpectralExtraction(PluginTemplateMixin):
             self.update_marks(step='ext')
 
         trace = self._get_ext_trace()
-        inp_sp2d = self._get_ext_input_spectrum().flux.value
+        inp_sp2d = self._get_ext_input_spectrum()
 
-        return extract.BoxcarExtract(inp_sp2d, trace, width=self.ext_width)
+        if self.ext_type_selected == 'Boxcar':
+            ext = extract.BoxcarExtract(inp_sp2d.data, trace, width=self.ext_width)
+        elif self.ext_type_selected == 'Horne':
+            uncert = inp_sp2d.uncertainty if inp_sp2d.uncertainty is not None else VarianceUncertainty(np.ones_like(inp_sp2d.data))  # noqa
+            if not hasattr(uncert, 'uncertainty_type'):
+                uncert = StdDevUncertainty(uncert)
+            image = NDData(inp_sp2d.data, uncertainty=uncert)
+            ext = extract.HorneExtract(image, trace)
+        else:
+            raise NotImplementedError(f"extraction type '{self.ext_type_selected}' not supported")  # noqa
+
+        return ext
 
     def export_extract_spectrum(self, add_data=False, **kwargs):
         """
@@ -626,6 +832,7 @@ class SpectralExtraction(PluginTemplateMixin):
         """
         extract = self.export_extract(**kwargs)
         spectrum = extract.spectrum
+
         # Specreduce returns a spectral axis in pixels, so we'll replace with input spectral_axis
         # NOTE: this is currently disabled until proper handling of axes-limit linking between
         # the 2D spectrum image (plotted in pixels) and a 1D spectrum (plotted in freq or
