@@ -26,25 +26,42 @@ def test_plugin(specviz2d_helper):
     # create FlatTrace
     pext.trace_type_selected = 'Flat'
     pext.trace_pixel = 28
-    trace = pext.export_trace()
+    trace = pext.export_trace(add_data=True)
     assert isinstance(trace, tracing.FlatTrace)
     assert trace.trace_pos == 28
     trace.trace_pos = 27
     pext.import_trace(trace)
     assert pext.trace_pixel == 27
 
+    # offset existing trace
+    pext.trace_trace_selected = 'trace'
+    pext.trace_offset = 2
+    trace = pext.export_trace(add_data=True)  # overwrite
+    assert isinstance(trace, tracing.FlatTrace)
+
     # create KosmosTrace
+    pext.trace_trace_selected = 'New Trace'
     pext.trace_type_selected = 'Auto'
-    trace = pext.export_trace()
+    trace = pext.export_trace(add_data=True)
     assert isinstance(trace, tracing.KosmosTrace)
     assert trace.guess == 27
-    trace = pext.export_trace(trace_pixel=26)
+    trace = pext.export_trace(trace_pixel=26, add_data=False)
     assert trace.guess == 26
     trace.guess = 28
     pext.import_trace(trace)
     assert pext.trace_pixel == 28
 
+    # offset existing trace
+    pext.trace_trace_selected = 'trace'
+    pext.trace_offset = 2
+    trace = pext.export_trace(add_data=True)  # overwrite
+    assert isinstance(trace, tracing.ArrayTrace)
+
+    # 3 new trace objects should have been loaded and plotted in the spectrum-2d-viewer
+    assert len(sp2dv.figure.marks) == 14
+
     # interact with background section, check marks
+    pext.trace_trace_selected = 'New Trace'
     pext.trace_type_selected = 'Flat'
     pext.bg_separation = 5
     pext.bg_width = 3
@@ -73,8 +90,8 @@ def test_plugin(specviz2d_helper):
     assert len(bg.traces) == 1
     assert bg.traces[0].trace[0] == 28 + 4
     assert bg.width == 3
-    bg = pext.export_bg(bg_width=5)
-    assert bg.width == 5
+    bg = pext.export_bg(bg_width=3.3)
+    assert bg.width == 3.3
     bg.width = 4
     pext.import_bg(bg)
     assert pext.bg_width == 4
@@ -101,6 +118,14 @@ def test_plugin(specviz2d_helper):
     sp_ext = pext.export_extract_spectrum()
     assert isinstance(sp_ext, Spectrum1D)
 
+    pext.ext_type_selected = 'Horne'
+    sp_ext = pext.export_extract_spectrum()
+    assert isinstance(sp_ext, Spectrum1D)
+
+    # test API calls
+    for step in ['trace', 'bg', 'ext']:
+        pext.update_marks(step)
+
     # test exception handling
     pext.trace_type = 'Auto'
     pext.bg_type_selected = 'TwoSided'
@@ -111,14 +136,34 @@ def test_plugin(specviz2d_helper):
     pext.vue_create_bg_img()
     assert 'should not be created' not in [d.label for d in specviz2d_helper.app.data_collection]
 
-    # test API calls
-    for step in ['trace', 'bg', 'ext']:
-        pext.update_marks(step)
-
     with pytest.raises(ValueError):
         pext.export_extract(invalid_kwarg=5)
 
 
+@pytest.mark.remote_data
+@pytest.mark.filterwarnings('ignore')
+def test_user_api(specviz2d_helper):
+    fn = download_file('https://stsci.box.com/shared/static/exnkul627fcuhy5akf2gswytud5tazmw.fits', cache=True)  # noqa
+
+    specviz2d_helper.load_data(spectrum_2d=fn)
+
+    pext = specviz2d_helper.plugins['Spectral Extraction']
+    pext.open_in_tray()
+
+    # test that setting a string to an AddResults object redirects to the label
+    pext.bg_sub_add_results = 'override label'
+    assert pext.bg_sub_add_results.label == 'override label'
+    pext.bg_sub_add_results.label = 'override label 2'
+    assert "override label 2" in pext.bg_sub_add_results.__repr__()
+    assert "override label 2" in pext.bg_sub_add_results._obj.auto_label.__repr__()
+
+    pext.export_bg_sub(add_data=True)
+    assert 'override label 2' in pext.ext_dataset.choices
+
+    # test setting auto label
+    pext.bg_sub_add_results.auto = True
+
+
 @pytest.mark.remote_data
 def test_spectrum_on_top(specviz2d_helper):
     fn = download_file('https://mast.stsci.edu/api/v0.1/Download/file/?uri=mast:jwst/product/jw01529-c1002_t002_miri_p750l_s2d.fits', cache=True)  # noqa
