@@ -16,9 +16,11 @@ from glue.core import HubListener
 from glue.core.edit_subset_mode import NewMode
 from glue.core.message import SubsetCreateMessage, SubsetDeleteMessage
 from glue.core.subset import Subset, MaskSubsetState
+from ipywidgets.widgets import widget_serialization
 
 from jdaviz.app import Application
 from jdaviz.core.events import SnackbarMessage
+from jdaviz.core.template_mixin import show_widget
 
 __all__ = ['ConfigHelper', 'ImageConfigHelper']
 
@@ -69,9 +71,24 @@ class ConfigHelper(HubListener):
             if hasattr(viewer, method):
                 getattr(viewer, method)(msg)
 
-    def load_data(self, data, parser_reference=None, **kwargs):
+    def load_data(self, data, data_label=None, parser_reference=None, **kwargs):
+        if data_label:
+            kwargs['data_label'] = data_label
         self.app.load_data(data, parser_reference=parser_reference, **kwargs)
 
+    @property
+    def plugins(self):
+        """
+        Access API objects for plugins in the plugin tray.
+
+        Returns
+        -------
+        plugins : dict
+            dict of plugin objects
+        """
+        return {item['label']: widget_serialization['from_json'](item['widget'], None).user_api
+                for item in self.app.state.tray_items}
+
     @property
     def fitted_models(self):
         """
@@ -278,7 +295,7 @@ class ConfigHelper(HubListener):
 
         return parameters_cube
 
-    def show(self, loc="inline", title=None):  # pragma: no cover
+    def show(self, loc="inline", title=None, height=None):  # pragma: no cover
         """Display the Jdaviz application.
 
         Parameters
@@ -321,56 +338,23 @@ class ConfigHelper(HubListener):
 
             NOTE: Only applicable to a "sidecar" display.
 
+        height: int, optional
+            The height of the top-level application widget, in pixels. Applies to all
+            instances of the same application in the notebook.
+
         Notes
         -----
         If "sidecar" is requested in the "classic" Jupyter notebook, the app will appear inline,
         as only JupyterLab has a mechanism to have multiple tabs.
         """
-        from IPython import get_ipython
-        from IPython.display import display
+        title = self.app.config if title is None else title
+        if height is not None:
+            if isinstance(height, int):
+                height = f"{height}px"
+            self.app.layout.height = height
+            self.app.state.settings['context']['notebook']['max_height'] = height
 
-        # Check if the user is running Jdaviz in the correct environments.
-        # If not, provide a friendly msg to guide them!
-        cur_shell_name = get_ipython().__class__.__name__
-        if cur_shell_name != 'ZMQInteractiveShell':
-            raise RuntimeError("\nYou are currently running Jdaviz from an unsupported "
-                               f"shell ({cur_shell_name}). Jdaviz is intended to be run within a "
-                               "Jupyter notebook, or directly from the command line.\n\n"
-                               "To run from Jupyter, call <your viz>.show() from a notebook cell.\n"
-                               "To see how to run from the command line, run: "
-                               "'jdaviz --help' outside of Python.\n\n"
-                               "To learn more, see our documentation at: "
-                               "https://jdaviz.readthedocs.io")
-
-        if loc == "inline":
-            display(self.app)
-
-        elif loc.startswith('sidecar'):
-            from sidecar import Sidecar
-
-            # Use default behavior if loc is exactly 'sidecar', else split anchor from the arg
-            anchor = None if loc == 'sidecar' else loc.split(':')[1]
-
-            # If title unset, default to the viz config
-            title = self.app.config if title is None else title
-
-            scar = Sidecar(anchor=anchor, title=title)
-            with scar:
-                display(self.app)
-
-        elif loc.startswith('popout'):
-            anchor = None if loc == 'popout' else loc.split(':')[1]
-
-            # Default behavior (no anchor specified): display popout in new window
-            if anchor in (None, 'window'):
-                self.app.popout_button.open_window()
-            elif anchor == "tab":
-                self.app.popout_button.open_tab()
-            else:
-                raise ValueError("Unrecognized popout anchor")
-
-        else:
-            raise ValueError(f"Unrecognized display location: {loc}")
+        show_widget(self.app, loc=loc, title=title)
 
     def show_in_sidecar(self, anchor=None, title=None):  # pragma: no cover
         """
@@ -699,8 +683,19 @@ def _next_subset_num(label_prefix, subset_groups):
     """Assumes ``prefix i`` format.
     Does not go back and fill in lower but available numbers. This is consistent with Glue.
     """
-    labels = [sg.label.split(' ')[1] for sg in subset_groups
-              if sg.label.startswith(label_prefix)]
-    if len(labels) == 0:
-        return 1
-    return max(map(int, labels)) + 1
+    max_i = 0
+
+    for sg in subset_groups:
+        if sg.label.startswith(label_prefix):
+            sub_label = sg.label.split(' ')
+            if len(sub_label) > 1:
+                i_str = sub_label[-1]
+                try:
+                    i = int(i_str)
+                except Exception:  # nosec
+                    continue
+                else:
+                    if i > max_i:
+                        max_i = i
+
+    return max_i + 1
